{

=pod

=head1 NAME

Userland::weblogUpdates

=head1 SUMMARY

 my $userland = Userland::weblogUpdates->new(transport=>SOAP,debug=>0);
 $userland->ping("Perlblog","http://www.nospum.net/perlblog");

=head1 DESCRIPTION

This package implements methods supported by the Userland weblogUpdates framework, for the weblogs.com website. 

=cut

package Userland::weblogUpdates;
use strict;

use vars qw ($VERSION);
$VERSION = 0.2;

use constant HOST  => "http://rpc.weblogs.com";
use constant PATH  => "/RPC2";
use constant CLASS => "weblogUpdates";
use constant PING  => "ping";

use Carp;

=pod

=head1 CONSTRUCTOR METHODS

=cut

=pod

=head2 $pkg = Userland::weblogUpdates->new(%args)

Valid arguments are 

=over 4

=item transport

I<string>. Valid transports are SOAP and XMLRPC. This field is required.

=item debug 

I<boolean>. Enable transport-specific debugging.

=back

=cut

sub new {
    my $pkg = shift;
    
    my $self = {};
    bless $self;

    $self->init(@_) || return undef;
    return $self;
}

sub init {
    my $self = shift;
    my $args = { @_ };

    if (! $args->{'transport'}) {
	carp "You must specify a transport.";
	return 0;
    }

    $self->Transport($args->{'transport'},debug=>$args->{'debug'}) 
	|| return 0;

    return 1;
}


=pod

=head1 API SPECIFIC METHODS

=cut

=pod

=head2 $pkg->ping($name,$uri)

Ping the Userland servers and tell them your weblog has been updated.

Returns true or false. This means that, unlike the Userland server itself, a successful ping returns 1 and a failed ping returns 0.

=cut

sub ping {
    my $self = shift;
    my $name = shift;
    my $url  = shift;

    if ((! $name) || (! $url)) {
	carp "You must specify both a weblog name and url";
	return 0;
    }

    my $meth = undef;
    my @args = ();

    if ($self->Transport() eq "xmlrpc") {
	$meth = join(".",CLASS,PING);
	@args = (
		 $self->_client()->string($name),
		 $self->_client()->string($url),
		 );
    }

    if ($self->Transport() eq "soap") {
	$meth = PING;
	@args = (
	       SOAP::Data->name(weblogname=>$name),
	       SOAP::Data->name(weblogurl=>$url),		 
		 );

    }

    if (! $meth) {
	carp "Unable to determine transport and method.";
	return 0;
    }

    my $res = $self->_do($meth,@args)
	|| &{ carp "Returned undef. Not good."; return 0; };

    return 1 if (! $res->{'flerror'});

    carp "Failed to ping : $res->{'message'}";
    return 0;
}

=pod

=head1 UTILITY METHODS

=cut

=pod

=head2 $pkg->Transport($transport,%args)

Set the transport for use with the package. Valid transports are SOAP and XMLRPC. This field is required.

Valid arguments are 

=over 4

=item debug 

I<boolean>. Enable transport-specific debugging.

=back

=cut

sub Transport {
    my $self      = shift;
    my $transport = shift;
    my $args      = { @_ };

    if (defined $transport) {

	if (! $transport =~ /^(xmlrpc|soap)$/i) {
	    delete $self->{"_transport"};
	    return undef;
	}

	$self->{"_transport"} = lc $transport;
	
	if (! $self->_client(debug=>$args->{'debug'})) {
	    delete $self->{"_transport"};
	    return undef;
	}

    }

    return $self->{"_transport"};
}

=pod

=head1 PRIVATE METHODS

=cut

sub _do {
    my $self = shift;
    my $meth = shift;
    my @args = @_;

    my $res = undef;

    if ($self->Transport() eq "xmlrpc") {
	eval { $res = $self->_client()->call($meth,@args); };

	if ($@) {
	    carp $@;
	    return 0;
	}

	# Hack.
	if ($res->{'flerror'}) {
	    $res->{'flerror'} = $res->{'flerror'}->value();
	}
    }

    # We don't bother wrapping this in an eval block
    # since we've already set a fault method for the 
    # SOAP::Lite object.

    if ($self->Transport() eq "soap") {
	$res = $self->_client()->call($meth,@args)->result();
    }

    return $res;
}

sub _client {
    my $self   = shift;
    my $client = "_".$self->Transport();
    return $self->$client(@_);
}

sub _xmlrpc {
    my $self = shift;
    my $args = { @_ };

    if (! $self->{"_xmlrpc"}) {

	my $class = "Frontier::Client";
	&_require($class) || return 0;

	$self->{"_xmlrpc"} = $class->new(url=>HOST.PATH,debug=>$args->{'debug'})
	    || &{ carp $!; return 0; };
    }

    return $self->{"_xmlrpc"};
}

sub _soap {
    my $self = shift;
    my $args = { @_ };

    if (! $self->{"_soap"}) {

	my $class = "SOAP::Lite";
	&_require($class) || return 0;
	
	my $soap = $class->new() ||
	    &{ carp $!; return 0; };
	
	# Where are we going?
	$soap->proxy(join("/",HOST,CLASS));

	# What are we doing?
	$soap->on_action(
			 sub{ 
			     "\"/".CLASS."\"" 
			     }
			 );

	# What if it doesn't work?
	$soap->on_fault(
			sub{ 
			    my ($soap,$res) = @_; 
			    carp (ref $res) ? $res->faultstring : $soap->transport->status(); 
			    return 0; 
			}
			);
	
	# Who's on first?
	if ($args->{'debug'}) {
	    $soap->on_debug(sub { print @_; });
	}

	$self->{"_soap"} = $soap;
    }

    return $self->{"_soap"};
}

sub _require {
    my $class = shift;
    
    eval "require $class" ||
	&{ carp $@; return 0; };
    
    return 1;
}

sub DESTROY {
    return 1;
}

=pod

=head1 TO DO 

=over 4

=item

Better error handling.

=item 

POD for private methods.

=back

=head1 VERSION

0.2

=head1 DATE

October 02, 2001

=head1 CHANGES

=head2 0.2

=over 4

=item

Added SOAP support.

=item

Added public I<transport> method.

=item

Added private I<_require>, I<_soap>, I<_xmlrpc> methods to load transport specific package(s) only when needed.

=item 

Added private I<_client> and I<_do> methods for performing transport specific actions.

=item 

Set the proper package name. I<gah!>

=item

Updated POD.

=back

=head2 0.1.1

=over 4

=item 

Added $VERSION (doh!)

=item

Updated POD

=item

Updated the Userland reference link

=back

=head2 0.1

=over 4

=item

Initial revision

=back

=head1 SEE ALSO

L<Frontier::Client>

L<SOAP::Lite>

http://www.weblogs.com

http://www.xmlrpc.com/discuss/msgReader$2014?mode=day

=head1 LICENSE

Copyright 2001 Aaron Straup Cope.

This is free software, you may use it and distribute it under the same terms as Perl itself.

=cut

return 1;

}
